#!/usr/bin/python
# -*- coding: utf-8 -*-


from jsonrpc_client import RPCClient
from logging import getLogger

log = getLogger("portable")

_opid_search_first2 = {
    "et": 232,
    "tm": 205,
    "os": 213,
    "uf": 262,
}
_opid_search_map = {
    "skasms": 1001,
    "skbopr": 1002,
    "skpr": 1001,
}


class Portace(object):

    CZ_TM = _opid_search_first2["tm"]
    CZ_VF = _opid_search_first2["os"]
    CZ_O2 = _opid_search_first2["et"]
    CZ_UF = _opid_search_first2["uf"]

    SK_ASMS = _opid_search_map["skasms"]
    SK_BO = _opid_search_map["skbopr"]

    DEFAULT = 200

    @staticmethod
    def get_operator_by_opid(opid):
        if opid is None:
            return Portace.DEFAULT
        # endif
        op = _opid_search_first2.get(opid[0:2], None)
        if op:
            return op
        # endif
        op = _opid_search_map.get(opid, None)
        if op:
            return op
        # endif
        return Portace.DEFAULT
    # enddef

    def __init__(self, config, section="portable"):
        """
        Trida na ziskavani informaci o zpravach z portaci
        """
        self._config = config
        self._section = section

        host = self._config.get(self._section, "host", None)
        if host:
            self.client = RPCClient(
                self._config, section=self._section, status_as_attr=True
            )
        else:
            log.critical(
                "!!! Using default configuration for portable !!!"
                " (host=sms-portable.ats, port=2011)"
            )
            self.client = RPCClient(
                host="sms-portable.ats", port=2011, status_as_attr=True
            )
        # endif

    # enddef

    def get_opids(self, numbers, shortcode):
        rets = []
        for msisdn in numbers:
            res = self.client.get_opid(msisdn=msisdn, sc=shortcode)
            res.raise_for_status(200)
            rets.append(res)
        # endfor
        return tuple(rets)
    # enddef

    def get_oper_id(self, number):
        res = self.client.get_operator(msisdn=number)
        res.raise_for_status(200)
        oper_id = 0

        number_str = str(number)

        if res == "cz_tm":
            oper_id = 205
        elif res == "cz_vf":
            oper_id = 213
        elif res == "cz_o2":
            oper_id = 232
        elif res.startswith("cz_fix"):
            oper_id = 850
        elif res.startswith("cz"):
            oper_id = 200
        elif (
            number_str.startswith("420") and
            len(number_str) == 12 and
            number_str.isdigit()
        ):
            oper_id = 200
        elif res.startswith("sk"):
            oper_id = 301
        elif (
            number_str.startswith("421") and
            len(number_str) == 12 and
            number_str.isdigit()
        ):
            oper_id = 301
        elif (
            number_str.startswith("48") and
            len(number_str) == 11 and
            number_str.isdigit()
        ):
            oper_id = 401
        # endif

        return oper_id
    # enddef

# endclass

# eof
