from .config import load_config
from .decorators import cursor as cursor_decorator
try:
    import uwsgi
except ImportError:
    uwsgi = None
# endtry


class Server:
    """
    základní server
    """
    shared = {
        'config': None
    }

    duality_master = None

    other_servers = {}

    _enable_response_check = None

    def _get_server(self, name):
        return self.other_servers[name]

    def __new__(cls, *args, **kwargs):
        cls.config = cls.shared['config']
        return super(Server, cls).__new__(cls)

    def __init__(
        self,
        config_path=None,
        config=None,
        # compatibility
        configPath=None  # pylint: disable=invalid-name
    ):

        self.config_path = (
            config_path if config_path is not None else configPath
        )
        if getattr(self, "config", None) is None:
            self.config = config

    def _init_config(self):
        if self.config is not None:
            pass
        elif self.config_path:
            self.config = load_config(self.config_path)
        else:
            self.config = load_config()
        # endif
        if self.config and self._enable_response_check is None:
            cfg_val = self.config.get(
                "rpc", "response_check", None
            )
            if cfg_val in ("true", "1"):
                self._enable_response_check = True
            elif cfg_val in ("false", "0", "", None):
                self._enable_response_check = False
            elif cfg_val == "critical":
                self._enable_response_check = "critical"
            else:
                print(
                    f"!!! rpc:response_check config value {cfg_val!r} "
                    "is wrong use boolean or 'critical'"
                )
                self._enable_response_check = False
            # endif

    def do_postfork(self, **kwargs):
        if not kwargs.get("force_mule") and uwsgi and uwsgi.mule_id() != 0:
            # mula nepotrebuje provadet postfork
            return
        # endif

        self._init_config()

        if hasattr(self, "postfork"):
            self.postfork()

        if self.duality_master is None and self.config:
            self.duality_master = self.config.getboolean(
                "uwsgi", "duality_master", None
            )

    # pylint: disable=unused-argument
    def do_atexit(self, **kwargs):
        if hasattr(self, "at_exit"):
            self.at_exit()

    @cursor_decorator(db="mysql")
    def _method_alive_mysql(self, c):
        c.execute("select 1")
        v = c.fetchone()
        assert v and v[0] == 1, "select 1 nevratil 1"

    @cursor_decorator(db="postgre")
    def _method_alive_postgre(self, c):
        c.execute("select 1")
        v = c.fetchone()
        assert v and v[0] == 1, "select 1 nevratil 1"

# eof
