from logging import getLogger
cache_log = getLogger("jsonrpc.connectors.memcache")


class DummyCacheClient(object):
    def __init__(self, *args, **kwargs):
        pass
    # enddef

    def __getattr__(self, *args, **kwargs):
        return self.fake
    # enddef

    def fake(self, *args, **kwargs):
        cache_log.warning("Using dummy cache object")
        return
    # enddef

    def disconnect_all(self):
        pass
    # enddef

# endclass


try:
    import memcache
    cache_log.debug("Pouzivam memcached jako clienta")
    from hashlib import md5
except ImportError:
    cache_log.exception("Nemohu naimportovat python pyckage: memcache")
    CacheClient = DummyCacheClient

    def connect_memcache(*args, **kwargs):
        return DummyCacheClient()
else:
    class CacheClient(memcache.Client):
        def __init__(self, *args, **kwargs):
            self._cache_key_prefix = kwargs.pop("prefix", None)
            if not self._cache_key_prefix:
                cache_log.warning(
                    "Cache key prefix NOT SET - may leed to cache cross!")
            # enddef
            memcache.Client.__init__(self, *args, **kwargs)
#            super(CacheClient).__init__(*args, **kwargs)
        # enddef

        def __alter_key(self, s_key):
            s_key = s_key.encode("utf8")
            key = md5(s_key).hexdigest()
            if self._cache_key_prefix:
                key = self._cache_key_prefix + key
            # endif
            # cache_log.debug("Key translate:%s=>%s", s_key, key)
            return key
        # enddef

        # @group Insertion: set, add, replace, set_multi

        def set(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            # endif
            return memcache.Client.set(self, key, *args, **kwargs)
        # enddef

        def add(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.add(self, key, *args, **kwargs)
        # enddef

        def replace(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.replace(self, key, *args, **kwargs)
        # enddef

        def set_multi(self, items, *args, **kwargs):
            items = dict((self.__alter_key(key), val)
                         for key, val in items.items())
            # endif
            return memcache.Client.set_multi(self, items, *args, **kwargs)
        # enddef

        # @group Retrieval: get, get_multi

        def get(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.get(self, key, *args, **kwargs)
        # enddef

        def get_multi(self, keys, *args, **kwargs):
            keys = tuple(self.__alter_key(key) for key in keys)
            return memcache.Client.get_multi(self, keys, *args, **kwargs)
        # enddef

        # @group Integers: incr, decr

        def incr(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.incr(self, key, *args, **kwargs)
        # enddef

        def decr(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.decr(self, key, *args, **kwargs)
        # enddef

        # @group Removal: delete, delete_multi

        def delete(self, key, *args, **kwargs):
            key = self.__alter_key(key)
            return memcache.Client.delete(self, key, *args, **kwargs)
        # enddef

        def delete_multi(self, keys, *args, **kwargs):
            keys = tuple(self.__alter_key(key) for key in keys)
            return memcache.Client.delete_multi(self, keys, *args, **kwargs)
        # enddef

        def disconnect_all(self):
            return memcache.Client.disconnect_all(self)
    # endclass

    def connect_memcache(config, section="memcache"):
        """
        helper funkce pro napojeni memcache
            Host=       #server na kterem to bezi
            Port=       #port kde to bezi
            Debug=      #jestli použít debug (hodnota 0/1)
        """
        server = config.get(section, "host", config.get(
            section, "server", "localhost"))
        port = config.getint(section, "port", 11211)
        debug = config.getboolean(section, "debug", False)
        prefix = config.get(section, "prefix", None)
        dead_retry = config.getint(section, "dead_retry", 30)
        server_max_value_length = config.getint(
            section, "server_max_value_length", None
        )
        conn = CacheClient(
            ['%s:%d' % (server, port)], debug=debug, prefix=prefix,
            dead_retry=dead_retry,
            server_max_value_length=server_max_value_length
        )
        return conn

    # enddef connect_memcache
# endtry

# eof
