from logging import getLogger
log = getLogger("jsonrpc.connectors.mysql")

DEFAULT_CHARSET = "utf8"


def connect_mysql(
    config=None, section="mysql",
    host=None, user=None, password=None, database=None, port=None, charset=None
):
    """
    helper funkce pro napojeni databaze
        Host=       #server na kterem to bezi
        Port=       #port kde to bezi
        Username=   #jmeno uzivatele
        Password=   #heslo
        Database=   #nazev databaze
    """
    try:
        import pymysql as MySQLdb
    except ImportError:
        log.exception("Nemohu importovat pymysql")
        return
    # endtry

    if config:
        charset = config.get(section, "charset", DEFAULT_CHARSET)
        conn = MySQLdb.connect(
            host=config.get(
                section, "host", config.get(section, "hostname", "localhost")
            ),
            user=config.get(section, "username", "user"),
            password=config.get(section, "password", "pass"),
            database=config.get(section, "database", "db"),
            port=config.getint(section, "port", 3306),
            charset=charset,
        )
        conn.config_section = section
        conn.config_object = config
    else:
        charset = charset or DEFAULT_CHARSET
        conn = MySQLdb.connect(
            host,
            user,
            password,
            database,
            port or 3306,
            charset=charset
        )
        conn.connect_args = {
            "host": host,
            "user": user,
            "password": password,
            "database": database,
            "port": port or 3306,
            "charset": charset
        }
    if charset:
        cur = conn.cursor()
        cur.execute("set names %s" % (charset, ))
    conn.autocommit(False)
    return conn
# enddef connect_db

# eof
