import functools
import pickle as cPickle
import json2
from time import sleep
from types import FunctionType
from .helpers import CacheBlocked
from .helpers import BLOCK_CACHE_VALUE
from .helpers import CACHE_FLUSH_BEFORE, CACHE_RESAVE_AFTER, CACHE_DO_FLUSH
from ..wrapper_classes import make_response_obj
import traceback

from logging import getLogger
log = getLogger("jsonrpc.decorators.cached")
log_write = getLogger("jsonrpc.decorators.cached.write")


class MarshallerJsonRpcRaw(object):
    """
    Marshaller pro vystup jsonrpc metod vyuzivajici cache
    """
    @staticmethod
    def dumps(v):
        return json2.dumps(v)

    @staticmethod
    def loads(v):
        return make_response_obj(
            v,
            content_type="application/json",
            http_code=200,
            data_type="json"
        )
# endclass


class Cached(object):

    flushBeforeExecute = CACHE_FLUSH_BEFORE
    recacheAfterExecute = CACHE_RESAVE_AFTER
    flushAndReturn = CACHE_DO_FLUSH

    def __init__(
            self, expires=3600, conditionFunction=None,
            blocking_call_for=None, waiting_call_for=None, key=None,
            object_attr="cache", marshaller=None):
        self.object_attr = object_attr
        self.expires = expires
        self.key = key
        self.cond = conditionFunction
        self.blocking_call_for = blocking_call_for
        self.waiting_call_for = waiting_call_for
        if waiting_call_for and blocking_call_for:
            log.warning(
                "waiting_call_for X blocking_call_for clash. Using wait")
            self.blocking_call_for = None
        self.marshaller = marshaller or cPickle
    # enddef __init__

    def _set(self, cacheObject, key, val, time):
        time = time if time is not None else 3600
        return cacheObject.set(
            key, val,
            time=time
        )

    def _delete(self, cacheObject, key):
        return cacheObject.delete(key)

    def _get(self, cacheObject, key):
        return cacheObject.get(key)

    def _get_cache_object(self, args):
        return getattr(args[0], self.object_attr, None)
    # enddef

    def __call__(self, func):
        """
        Pozor!!! cache key obsahuje jen kw_args
        """
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            nf_reason = None
            log.debug(
                "cashing function %s with args:%s and cache_args:%s" % (
                    func.__name__, args, kwargs))
            # args[0] je objekt ktery by mel obsahovat cache (u tridnich metod
            #   je to self)
            # ted si overim ze tomu tak je a najdu si ten cache object
            cacheObject = self._get_cache_object(args)

            doFlush = str(kwargs.pop("__FLUSH", ""))

            if not cacheObject:
                # nemám cache ale já se nesložím ;-) proste jen necahuju
                log.warning("Unusable cache!! on %s", type(args[0]))
                return func(*args, **kwargs)
            # endif

            # tohle musi byt zde jinak se __FLUSH dostane do cachekeys a cely
            #   to nefunguje
            if self.key:
                cache_key = self.key(func, args, kwargs)
            else:
                cache_key = "_".join(
                    [args[0].__class__.__name__, func.__name__] + [
                        repr(i) for i in sorted(kwargs.items())
                    ]
                )
                cache_key = cache_key.replace(" ", "")
            # endif
            log.debug("%s CACHE KEY:%s", doFlush, cache_key)
            # dulezita poznamka standartni memcache podle vseho ukrajuje klice
            #   delsi nez 250 znaku takze to tu musim resit... a to tak ze
            #   zaloguju a nepouziju cache a to proste protoze lepsi necahovat
            #   nez cachovat blbě!!!

            # uz neni potreba delata resi to wrapper na memcache.
#
#            if len(cache_key) > 250:
#                self.logger.warning(
#                    "Cache key je vetsi nez 250B to je moc! necachuju :-!")
#                return func(*args, **kwargs)
#            # endif

            # ted checknu jestli nahodou nemam udelat preflush cache
            if doFlush in (self.flushBeforeExecute, self.flushAndReturn):
                try:
                    res = self._delete(cacheObject, cache_key)
                    log.warning("%symazano (res:%s) z cache:%s" % (
                        "Nev" if res != 1 else "V", res, cache_key))
                    log_write.debug(
                        "write flush ok func: %s, key: %s, reason: %s",
                        func.__name__, cache_key, doFlush
                    )
                except BaseException as e:
                    # nepodarilo se smazat z cache (cokoliv se uz stalo),
                    #   takze zaloguju a pokracuju vesele dal be cache
                    msg = "Chyba vymazani dat z cache(%s):%r%s"
                    log.warning(msg, cache_key, e, e)
                    log.debug(msg + "\n%s",
                              cache_key, e, e, traceback.format_exc())
                    log_write.debug(
                        "write flush err1 func: %s, key: %s, reason: %s, "
                        "e: %r",
                        func.__name__, cache_key, doFlush, e
                    )
                # endtry
                if doFlush == self.flushAndReturn:
                    return
                # endif
            # endif

            if (
                self.cond is not None and
                not self.cond(*args, __function_name=func.__name__, **kwargs)
            ):
                log.debug(
                    "Caching of %s skipped due to condition %s",
                    func.__name__, self.cond.__name__)
                return func(*args, **kwargs)
            # endif
            cached = False
            if doFlush not in (
                    self.recacheAfterExecute, self.flushBeforeExecute):
                try:
                    cached = self._get(cacheObject, cache_key)
                except BaseException as e:
                    # nepodarilo se zskat z cache (cokoliv se uz stalo), takze
                    #   zaloguju a pokracuju vesele dal be cache
                    log.warning(
                        "Chyba ziskani dat z cache(%s):%s%s" % (
                            cache_key, type(e), e))
                    log_write.debug(
                        "write load err2 func: %s, key: %s, reason: %s, "
                        "e: %r",
                        func.__name__, cache_key, doFlush, e
                    )
                    cached = False
                # endtry
            # endif
            if cached:
                if cached == BLOCK_CACHE_VALUE:
                    if self.waiting_call_for:
                        for i in range(1, self.waiting_call_for):
                            sleep(1)
                            try:
                                cached = self._get(cacheObject, cache_key)
                                if cached and cached != BLOCK_CACHE_VALUE:
                                    # uz to prislo -> vyskoč
                                    break
                            except BaseException as e:
                                log.warning(
                                    "Chyba ziskani dat z cache(%s):%s%s" % (
                                        cache_key, type(e), e))
                                raise CacheBlocked()
                            # endtry
                        else:
                            # nedockal jsem se  –> konec :-(
                            raise CacheBlocked()
                        # endif
                    else:
                        raise CacheBlocked()
                    # endif
                try:
                    res = self.marshaller.loads(cached)
                    log.debug("From cache %s", cache_key)
                    return res
                except BaseException as e:
                    # nepodarilo se nacist data z cache (cokoliv se uz stalo),
                    #   takze zaloguju a pokracuju vesele dal be cache
                    log.warning(
                        "Chyba nacteni dat z cache:%s%s" % (type(e), e))
                    log_write.debug(
                        "write load err3 func: %s, key: %s, e: %r",
                        func.__name__, cache_key, e
                    )
                # endtry
            # endif
            if self.blocking_call_for is not None:
                cwres = self._set(
                    cacheObject, cache_key,
                    BLOCK_CACHE_VALUE,
                    time=self.blocking_call_for
                )
                log_write.debug(
                    "write ok1 func: %s, key: %s, size: %s, expires: %s, "
                    "cache: %s, res: %r, nf_reason: %r",
                    func.__name__, cache_key, len(BLOCK_CACHE_VALUE),
                    self.expires, cacheObject.__class__.__name__, cwres,
                    nf_reason,
                )
            elif self.waiting_call_for is not None:
                cwres = self._set(
                    cacheObject, cache_key,
                    BLOCK_CACHE_VALUE,
                    time=self.waiting_call_for
                )
                log_write.debug(
                    "write ok2 func: %s, key: %s, size: %s, expires: %s, "
                    "cache: %s, res: %r, nf_reason: %r",
                    func.__name__, cache_key, len(BLOCK_CACHE_VALUE),
                    self.expires, cacheObject.__class__.__name__, cwres,
                    nf_reason,
                )

            # endif

            res = func(*args, **kwargs)
            try:
                m_data = self.marshaller.dumps(res)
                cwres = self._set(
                    cacheObject, cache_key, m_data, time=self.expires
                )
                log_write.debug(
                    "write ok3 func: %s, key: %s, size: %s, expires: %s, "
                    "cache: %s, res: %r, nf_reason: %r",
                    func.__name__, cache_key, len(m_data), self.expires,
                    cacheObject.__class__.__name__, cwres, nf_reason,
                )
            except BaseException as e:
                # nepodarilo se zapsat do cache (cokoliv se uz stalo), takze
                #   zaloguju a pokracuju vesele dal be cache
                msg = "Chyba ulozeni dat do cache(%s):%r%s"
                log.warning(msg, cache_key, e, e)
                log.debug(msg + "\n%s",
                          cache_key, e, e, traceback.format_exc())
                log_write.debug(
                    "write err func: %s, key: %s, expires: %s, error: %s, "
                    "cache: %s",
                    func.__name__, cache_key, self.expires, e,
                    cacheObject.__class__.__name__
                )
            # endtry
            return res
        # enddef wrapper
        wrapper._wraps = getattr(func, "_wraps", func)
        return wrapper
    # enddef __call__
# endclass Cached


def cached(
    expires=3600, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache", marshaller=None
):
    """
    dekorator pro cache
    """
    # pokud se jedna o volani bez parametru pomoci @cached tak je v expires
    #   ta funkce pokud ovšem ne tak se jedná o volání typu s paramatry
    #   (třeba @cached(expires=8000) ) a pak chci vrátit správně upravený
    #   ten cached objekt
    if isinstance(expires, FunctionType):
        # jedna se o prime volani jako funkce a ne jako trida
        return Cached()(expires)
    # endif
    return Cached(
        expires, conditionFunction, blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for, key=key, object_attr=attr,
        marshaller=marshaller
    )
# enddef cached


def cached_json2(
    expires=3600, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache"
):
    return cached(
        expires=expires,
        conditionFunction=conditionFunction,
        blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for,
        key=key,
        attr=attr,
        marshaller=json2
    )
# enddef


def cached_rpc_raw_response(
    expires=3600, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache"
):
    return cached(
        expires=expires,
        conditionFunction=conditionFunction,
        blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for,
        key=key,
        attr=attr,
        marshaller=MarshallerJsonRpcRaw
    )
# enddef


def not_cached(func):
    """
    osetreni na necachovanou metodu pokud muze dostat __FLUSH,
    tak aby to ignorovala
    """

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        kwargs.pop("__FLUSH", None)
        return func(*args, **kwargs)
    # enddef
# enddef

# eof
