import traceback
import functools
from logging import getLogger, DEBUG

from types import FunctionType

# from types import GeneratorType
from ..wrapper_classes import generatorTypes, CursorGeneratorWrapper

from .helpers import psycopg2
from .helpers import DBTYPE_MYSQL, DBTYPE_PSQL

try:
    import pymysql as MySQLdb
    import pymysql.cursors as MySQLCursors
except ImportError:
    print("Nepodarilo se importovat pymysql")
    raise


from ..connectors import connect_db, connect_postgre


log = getLogger("jsonrpc.decorators.cursor")
info_log = getLogger("jsonrpc.decorators.cursor_info")
commmit_log = getLogger("jsonrpc.decorators.cursor.commit")

PgRealDictCursor = psycopg2.extras.RealDictCursor
iog = getLogger("jsonrpc.decorators.cursor")
PgNamedTupleCursor = psycopg2.extras.NamedTupleCursor
PgMyNamedTupleCursor = psycopg2.extras.MyNamedTupleCursor


# pylint: disable=too-few-public-methods
class Cursor:
    # pylint: disable=too-many-arguments
    def __init__(
        self,
        dict_result=False,
        readonly=False,
        db=None,
        real_dict_cursor=False,
        db_attr=None,
        db_attr_ro=None,
    ):
        if real_dict_cursor and not dict_result:
            e_msg = (
                "Cursor is set to be 'real_dict_cursor' "
                "but 'dict_result' is not set. "
                "Dunno if this will break anything, check it please.\n"
                "If it breaks, add True as a first position arg to cursor"
            )
            if log.isEnabledFor(DEBUG):
                tb = "\n".join(traceback.format_stack())
                log.critical("%s\n%s", e_msg, tb)
            else:
                log.critical(e_msg)

            dict_result = True

        self._dict_result = dict_result
        self._readonly = readonly  # TODO
        self._real_dict_cursor = real_dict_cursor
        self._db = db
        if self._db not in (None, False, DBTYPE_MYSQL, DBTYPE_PSQL):
            msg = f"Unknown database type {self._db}"
            log.critical(msg)
            raise AttributeError(msg)
        # endif
        if db_attr:
            self._db_attr = db_attr
        else:
            self._db_attr = self._db

        if not db_attr_ro:
            if self._db_attr:
                db_attr_ro = "{self._db_attr}_ro"
            elif self._db:
                db_attr_ro = "{self._db}_ro"
            # endif
        # endif

        self._db_attr_ro = db_attr_ro
        if self._db is None:
            # CO TO PROBOHA JE?? pada to - sprav si to ;-)
            # hraj si na svym serveru :–P
            # Poznamka: komentare zachovany pro budouci generace, kod presunut
            pass

    # enddef

    # pylint: disable=too-many-branches,too-many-statements
    def __call__(self, func):
        # TODO: zvazit predavani cursor_reuse a commitovat pouze pri
        # not cursor_reuse

        # pylint: disable=too-many-locals
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            kw_cursor = kwargs.pop("cursor", None)
            cursor_reuse = kw_cursor is not None
            close_when_done = True
            if not self._db:
                if hasattr(args[0], DBTYPE_PSQL):
                    self._db = DBTYPE_PSQL
                else:
                    self._db = DBTYPE_MYSQL
                # endif
                if not self._db_attr:
                    self._db_attr = self._db
                if not self._db_attr_ro:
                    self._db_attr_ro = self._db_attr + "_ro"
            # endif
            log_conn_from_attr = None
            if cursor_reuse:
                log.debug("reusing cursor from args with type %s", self._db)
                # kontrola kurzoru
                test_class = None
                if self._db == DBTYPE_PSQL:
                    conn = kw_cursor.connection
                    if self._dict_result:
                        if self._real_dict_cursor:
                            test_class = (PgRealDictCursor,)
                        else:
                            test_class = (
                                PgNamedTupleCursor,
                                PgMyNamedTupleCursor,
                            )
                    else:
                        test_class = None  # TODO
                    # endif
                elif self._db == DBTYPE_MYSQL:
                    conn = None
                    if self._dict_result:
                        test_class = (MySQLCursors.DictCursor,)
                    else:
                        test_class = (MySQLCursors.Cursor,)
                    # endif
                # endif

                # tady nemuzu pouzit isinstance protoze dictcursory jsou
                # potomky cursoru
                if test_class and not isinstance(kw_cursor, test_class):
                    log.fatal(
                        "Posilas chybny typ kurzoru pro reuse %s - "
                        "ocekavam %s\n%s",
                        type(kw_cursor),
                        test_class,
                        "".join(traceback.format_stack()),
                    )
                    # TODO kvuli zpetne kompatibilite jen loguji, ale chtelo
                    # by to:
                    # raise ProgrammingError("chybny typ kurzoru")
                # endif

                if self._db == DBTYPE_PSQL:
                    if (
                        self._readonly is False
                        and kw_cursor.connection.readonly is True
                    ):
                        # ja potrebuji zapisovaci kurzor, ale prichazi mi
                        # pouze cteci kurzor => nelibi se mi to
                        # v opacnem pripade mi to nevadi (potrebuji cteci ale
                        # prichazi zapisovaci)
                        log.fatal(
                            "Posilas chybny typ kurzoru pro reuse r/w %s - "
                            "ocekavam %s\n%s",
                            kw_cursor.connection.readonly,
                            test_class,
                            "".join(traceback.format_stack()),
                        )
                    # endif
                # endif
            else:
                conn_from_attr = None
                if (
                    self._readonly
                    and self._db_attr_ro
                    and hasattr(args[0], self._db_attr_ro)
                ):
                    conn_from_attr = self._db_attr_ro
                    log_conn_from_attr = "_db_attr_ro"
                else:
                    conn_from_attr = self._db_attr
                    log_conn_from_attr = "_db_attr"
                conn = getattr(args[0], conn_from_attr, None)
                if conn is None:
                    raise RuntimeError(
                        f"{type(args[0])} attribute {conn_from_attr} which "
                        "should contain database connection is not available"
                    )
                try:
                    test_cur = conn.cursor()
                    test_cur.execute("select 1")
                    test_cur.close()
                except (
                    MySQLdb.ProgrammingError,
                    MySQLdb.OperationalError,
                    MySQLdb.InterfaceError,
                    MySQLdb.DatabaseError,
                ) as e:
                    if self._db != DBTYPE_MYSQL:
                        raise
                    log.warning("Mysql error on check:%s:%s", type(e), e)
                    if hasattr(conn, "config_object"):
                        conn = connect_db(
                            conn.config_object, section=conn.config_section
                        )
                    elif hasattr(conn, "connect_args"):
                        conn = connect_db(**conn.connect_args)
                    elif hasattr(args[0], "config"):
                        conn = connect_db(
                            args[0].config,
                            section=getattr(
                                conn,
                                "config_section",
                                self._db_attr or self._db,
                            ),
                        )
                    else:
                        log.critical(
                            "Can't reconnect database!!! I have no config "
                            "attribute on object %s",
                            type(args[0]),
                        )
                        raise
                    # endif
                    setattr(args[0], conn_from_attr, conn)
                except (
                    psycopg2.OperationalError,
                    psycopg2.InterfaceError,
                    psycopg2.InternalError,
                    psycopg2.DatabaseError,
                ) as e:
                    if self._db != DBTYPE_PSQL:
                        raise
                    log.warning(
                        "Postgre error on check:%r:%r:%r", type(e), e, e.pgerror
                    )
                    if hasattr(conn, "connect_args"):
                        cargs = conn.connect_args
                        conn = connect_postgre(**cargs)
                        setattr(conn, "connect_args", cargs)
                    elif hasattr(conn, "config_object"):
                        info_log.debug(
                            "New connecion from config_object, section=%r",
                            conn.postgre_config_section,
                        )
                        conn = connect_postgre(
                            conn.config_object,
                            section=conn.postgre_config_section,
                        )
                    elif hasattr(args[0], "config"):
                        info_log.debug(
                            "New connection from config, section=%r",
                            args[0].config.postgre_config_section,
                        )
                        conn = connect_postgre(
                            args[0].config,
                            section=args[0].config.postgre_config_section,
                        )
                    else:
                        log.critical(
                            "Can't reconnect database!!! I have no config "
                            "attribute on object %s",
                            conn,
                        )
                        raise
                    # endif
                    if self._readonly is False and conn.readonly is True:
                        # ja potrebuji zapisovaci kurzor, ale databaze po
                        # reconnectu je pouze cteci => nelibi se mi to
                        # v opacnem pripade mi to nevadi (potrebuji cteci ale
                        # prichazi zapisovaci)
                        log.fatal(
                            "Požaduješ write cursor ale po reconnectu je "
                            "connection jen readonly\n%s",
                            "".join(traceback.format_stack()),
                        )
                    # endif
                    if log.isEnabledFor(DEBUG):
                        try:
                            dsn = conn.get_dsn_parameters()
                        except BaseException:
                            log.exception("Dsn info not found")
                            dsn = {}
                        # endtry
                        info_log.debug(
                            (
                                "Conn set fn:%s reuse:%r, ro: %r, conn:(%r, %r"
                                " %r, ro=%r, attr=%r)"
                            ),
                            func.__name__,
                            bool(cursor_reuse),
                            bool(self._readonly),
                            dsn.get("host"),
                            dsn.get("dbname"),
                            dsn.get("user"),
                            conn.readonly,
                            log_conn_from_attr,
                        )
                    setattr(args[0], conn_from_attr, conn)
                except BaseException as e:
                    log.critical(
                        "Unknown exception %r, type: %r, dir: %r",
                        e,
                        type(e),
                        dir(e),
                    )
                    raise

                # endtry
                if self._db == DBTYPE_MYSQL:
                    cursor_class = (
                        MySQLCursors.DictCursor
                        if self._dict_result
                        else MySQLCursors.Cursor
                    )
                    kw_cursor = conn.cursor(cursor_class)
                else:
                    if self._dict_result:
                        if self._real_dict_cursor:
                            kw_cursor = conn.cursor(
                                cursor_factory=PgRealDictCursor
                            )
                        else:
                            kw_cursor = conn.cursor(
                                cursor_factory=PgMyNamedTupleCursor
                            )
                    else:
                        kw_cursor = conn.cursor()
            # endif
            if self._db == DBTYPE_PSQL and log.isEnabledFor(DEBUG):
                try:
                    dsn = kw_cursor.connection.get_dsn_parameters()
                except BaseException:
                    log.exception("Dsn info not found")
                    dsn = {}
                # endtry
                info_log.debug(
                    (
                        "Cursor fn:%s reuse:%r, ro: %r, conn:(%r, %r %r, "
                        "ro=%r, attr=%r)"
                    ),
                    func.__name__,
                    bool(cursor_reuse),
                    bool(self._readonly),
                    dsn.get("host"),
                    dsn.get("dbname"),
                    dsn.get("user"),
                    kw_cursor.connection.readonly,
                    log_conn_from_attr,
                )
            # endif
            args = list(args)
            args.insert(1, kw_cursor)
            try:
                res = func(*args, **kwargs)
                if isinstance(res, tuple(generatorTypes)):
                    close_when_done = False
                    res = CursorGeneratorWrapper(
                        kw_cursor, res, kw_cursor.connection, reuse=cursor_reuse
                    )
                # endif
            except MySQLdb.ProgrammingError as e:
                if self._db != DBTYPE_MYSQL:
                    if not cursor_reuse:
                        # pozor i rollback nad rozbitou DB muze vyvolat vyjimku
                        log.debug("ROLLBACK")
                        commmit_log.debug(
                            "Rollback\n%s!", traceback.format_stack()
                        )
                        conn.rollback()
                    raise
                # endif

                error_no = e.args[0]
                if error_no == 2014:
                    # "Commands out of sync; you can't run this command now"
                    # tohle zanamena ze se mi buh vi jak rozpadlo spojeni
                    # ja ho chci znovu nahodit takze reconnect - zvlastni je
                    # ze ve spouste pripadu jde udelat select true i nad takhle
                    # nakopnutou databazi. asi to souvisí s nevolámín
                    # selectu nad konkretní tabulkou
                    if not cursor_reuse:
                        # pokusim se o nejake uklizeni
                        try:
                            log.debug("ROLLBACK")
                            commmit_log.debug(
                                "Rollback\n%s!", traceback.format_stack()
                            )
                            conn.rollback()
                        except BaseException:
                            log.exception("Chyba rollbacku u error:2014")
                        # endtry
                        try:
                            conn.close()
                        except BaseException:
                            log.exception("Chyba close u error:2014")
                        # endtry

                        # reconnect
                        if hasattr(args[0], "config"):
                            conn = connect_db(
                                args[0].config, section=conn.config_section
                            )
                            args[0].mysql = conn
                        else:
                            log.critical(
                                "Can't reconnect database!!! I have no config "
                                "attribute on object %s",
                                type(args[0]),
                            )
                        # endif hasattr(config)
                    # endif cursor_reuse
                    raise
                qry = getattr(kw_cursor, "_last_executed", "<UNKNOWN>")
                log.exception("Error while executing query: \n %s", qry)
                if not cursor_reuse:
                    # pozor i rollback nad rozbitou DB muze vyvolat vyjimku
                    log.debug("ROLLBACK")
                    commmit_log.debug("Rollback\n%s!", traceback.format_stack())
                    conn.rollback()
                # endif
                raise
            except MySQLdb.Error:
                if self._db == DBTYPE_MYSQL:
                    qry = getattr(kw_cursor, "_last_executed", "<UNKNOWN>")
                    log.exception("Error while executing query: \n %s", qry)
                # endif
                # warning - pouziti nedokumentovaneho api ->
                # muze v budoucnu prinest problemy
                log.exception(
                    "Error while executing query: \n %s",
                    kw_cursor._last_executed,
                )
                if not cursor_reuse:
                    # pozor i rollback nad rozbitou DB muze vyvolat vyjimku!!
                    log.debug("ROLLBACK")
                    commmit_log.debug("Rollback\n%s!", traceback.format_stack())
                    conn.rollback()
                # endif
                raise
            except psycopg2.Error:
                if self._db == DBTYPE_PSQL:
                    log.exception(
                        "Error while executing query: \n %s", kw_cursor.query
                    )
                # endif
                if not cursor_reuse:
                    # pozor i rollback nad rozbitou DB muze vyvolat vyjimku!!
                    log.debug("ROLLBACK")
                    if conn.closed == 0:
                        commmit_log.debug(
                            "Rollback\n%s!", traceback.format_stack()
                        )
                        conn.rollback()
                    else:
                        log.warning(
                            "Can't rollback connection closed(%r)!!",
                            conn.closed,
                        )
                    # endif
                # endif
                raise
            except Exception as e:
                log.critical(
                    "Unknown exception %r, type: %r, dir: %r, reuse: %r",
                    e,
                    type(e),
                    dir(e),
                    cursor_reuse,
                )
                if not cursor_reuse:
                    # pozor i rollback nad rozbitou DB muze vyvolat vyjimku!!
                    log.debug("ROLLBACK")
                    try:
                        commmit_log.debug(
                            "Rollback\n%s!", traceback.format_stack()
                        )
                        conn.rollback()
                    except BaseException:
                        # pylint: disable=raise-missing-from
                        raise e
                # endif
                raise
            if not cursor_reuse and close_when_done:
                # pozor commit muze vyvolat vyjimku pri nesplneni treba
                # foreign keys!!
                log.debug("COMMIT")
                commmit_log.debug("Commit\n%s!", traceback.format_stack())
                conn.commit()
            # endif
            if not cursor_reuse and close_when_done:
                kw_cursor.close()
            # endif
            return res

        wrapper._wraps = getattr(func, "_wraps", func)
        return wrapper

    # enddef


# endclass


# pylint: disable=too-many-arguments
def cursor(
    dict_result=False,
    readonly=False,
    db=None,
    real_dict_cursor=False,
    db_attr=None,
    db_attr_ro=None,
):
    """
    dekorator pro databazi
    """
    if isinstance(dict_result, FunctionType):
        c = Cursor(
            readonly=readonly,
            db=db,
            real_dict_cursor=real_dict_cursor,
            db_attr=db_attr,
            db_attr_ro=db_attr_ro,
        )
        return c(dict_result)
    return Cursor(
        dict_result,
        readonly=readonly,
        db=db,
        real_dict_cursor=real_dict_cursor,
        db_attr=db_attr,
        db_attr_ro=db_attr_ro,
    )


# enddef cursor

# eof
