import functools
from types import FunctionType
import warnings
import inspect


class Deprecated:
    def __init__(self, replacement):
        self.replacement = replacement
    # enddef __init__

    def __call__(self, func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                message = "Method '%s' deprecated " % (func.__name__,)
                if self.replacement:
                    message += " use '%s' instead" % (self.replacement,)
                (frame, filename, line_number, function_name, lines,
                 index) = inspect.getouterframes(inspect.currentframe())[1]
                message += ". Called from %s:%d(%s:%s)" % (
                    filename, line_number, function_name, lines[0].strip())
                print(
                    frame, filename, line_number, function_name, lines, index)

                print("DEPRECATED: " + message)
                warnings.warn(message, DeprecationWarning, stacklevel=2)
            except BaseException as e:
                print(type(e), e)
            return func(*args, **kwargs)
        # enddef wrapper
        wrapper._wraps = getattr(func, "_wraps", func)
        return wrapper
    # enddef __call__
# endclass Deprecated


def deprecated(replacement):
    """
    dekorator pro deprecation warning
    """
    # pokud se jedna o volani bez parametru pomoci @deprecated tak je v
    #   replacement ta funkce pokud ovšem ne tak se jedná o volání typu
    #   s paramatry (třeba @deprecated(replacement) ) a pak chci vrátit správně
    #   upravený ten deprecated objekt
    if isinstance(replacement, FunctionType):
        # jedna se o prime volani jako funkce a ne jako trida
        return Deprecated(None)(replacement)
    # endif
    return Deprecated(replacement)
# enddef deprecated

# eof
