#!/usr/bin/python
# -*- coding: utf-8 -*-

import traceback
import functools
from logging import getLogger

from types import GeneratorType, FunctionType

from ..connectors.mysql_pool_conn import Pool
import pymysql.cursors as MySQLCursors


log = getLogger("jsonrpc.decorators.cursor")
info_log = getLogger("jsonrpc.decorators.cursor_info")
commmit_log = getLogger("jsonrpc.decorators.cursor.commit")


class Cursor(object):

    def __init__(self, dict_result=False, db_attr=None):
        self._dict_result = dict_result

        self._db_attr = db_attr or "mysql"
    # enddef

    def __call__(self, func):

        if isinstance(func, GeneratorType):
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                cursor_reuse = kwargs.get("cursor") is not None
                if cursor_reuse:
                    log.debug(
                        "reusing cursor from args"
                    )
                    cursor = kwargs.pop("cursor")
                    # kontrola kurzoru
                    test_class = None
                    if self._dict_result:
                        test_class = (MySQLCursors.DictCursor,)
                    else:
                        test_class = (MySQLCursors.Cursor,)
                    # endif

                    # tady nemuzu pouzit isinstance protoze dictcursory jsou
                    # potomky cursoru
                    if test_class and not isinstance(cursor, test_class):
                        log.fatal(
                            "Posilas chybny typ kurzoru pro reuse %s - "
                            "ocekavam %s\n%s",
                            type(cursor), test_class,
                            "".join(traceback.format_stack())
                        )
                        raise RuntimeError("chybny typ kurzoru")

                    args = list(args)
                    args.insert(1, cursor)
                    res = func(*args, **kwargs)
                    if isinstance(res, GeneratorType):
                        for i in res:
                            yield res
                    else:
                        yield res
                else:
                    pool = getattr(args[0], self._db_attr)
                    if not isinstance(pool, Pool):
                        raise TypeError(
                            "This is not mysql pool - %r on %r is %r" % (
                                self._db_attr, args[0], pool
                            )
                        )
                    cursor_class = (
                        MySQLCursors.DictCursor
                        if self._dict_result
                        else MySQLCursors.Cursor
                    )
                    with pool.get_cursor(cursor_class) as cur:
                        kwargs.pop("cursor", None)
                        args = list(args)
                        args.insert(1, cur)
                        res = func(*args, **kwargs)

                        if isinstance(res, GeneratorType):
                            for i in res:
                                yield res
                        else:
                            yield res
                        # endif
                    # endwith
                # endif
            # enddef
        else:
            @functools.wraps(func)
            def wrapper(*args, **kwargs):
                cursor_reuse = kwargs.get("cursor") is not None
                if cursor_reuse:
                    log.debug(
                        "reusing cursor from args"
                    )
                    cursor = kwargs.pop("cursor")
                    # kontrola kurzoru
                    test_class = None
                    if self._dict_result:
                        test_class = (MySQLCursors.DictCursor,)
                    else:
                        test_class = (MySQLCursors.Cursor,)
                    # endif

                    # tady nemuzu pouzit isinstance protoze dictcursory jsou
                    # potomky cursoru
                    if test_class and not isinstance(cursor, test_class):
                        log.fatal(
                            "Posilas chybny typ kurzoru pro reuse %s - "
                            "ocekavam %s\n%s",
                            type(cursor), test_class,
                            "".join(traceback.format_stack())
                        )
                        raise RuntimeError("chybny typ kurzoru")
                    args = list(args)
                    args.insert(1, cursor)
                    ret = func(*args, **kwargs)
                    return ret
                else:
                    pool = getattr(args[0], self._db_attr)
                    if not isinstance(pool, Pool):
                        raise TypeError(
                            "This is not mysql pool - %r on %r is %r" % (
                                self._db_attr, args[0], pool
                            )
                        )
                    cursor_class = (
                        MySQLCursors.DictCursor
                        if self._dict_result
                        else MySQLCursors.Cursor
                    )
                    with pool.get_cursor(cursor_class) as cur:
                        kwargs.pop("cursor", None)
                        args = list(args)
                        args.insert(1, cur)
                        ret = func(*args, **kwargs)
                        if ret is cur:
                            raise RuntimeError(
                                "Prosím nevracej kurzor timhle způsobem. "
                                "Buď ho vyčit nebi z funkce yielduj"
                            )
                        elif isinstance(ret, GeneratorType):
                            log.warning(
                                "Návratová hodnota je generator. To je "
                                "potencialně nebezpečné. Asi sem ti zavřel "
                                "kurzor pod rukama"
                            )
                            return ret
                        else:
                            return ret
                        # endif
                    # endwith
                # endif
            # enddef
        # endif - generator or function
        wrapper._wraps = getattr(func, "_wraps", func)
        return wrapper
    # enddef
# endclass


def cursor(dict_result=False, db_attr=None):
    """
    dekorator pro databazi
    """
    if isinstance(dict_result, FunctionType):
        c = Cursor(db_attr=db_attr)
        return c(dict_result)
    return Cursor(dict_result=dict_result, db_attr=db_attr)
# enddef cursor

# eof
