from types import FunctionType

from .cached import Cached, MarshallerJsonRpcRaw
import json2
from logging import getLogger
log = getLogger("jsonrpc.decorators.cached")


class RedCached(Cached):
    def __init__(self, expires=None, *args, **kwargs):
        super(RedCached, self).__init__(expires, *args, **kwargs)
    # enddef

    def _set(self, cacheObject, key, val, time):
        if not time:
            return cacheObject.set(key, val)
        else:
            return cacheObject.setex(key, time, val)
        # enddef
# endclass Cached


def red_cached(
    expires=None, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache", marshaller=None
):
    """
    dekorator pro cache
    """
    # pokud se jedna o volani bez parametru pomoci @cached tak je v expires
    #   ta funkce pokud ovšem ne tak se jedná o volání typu s paramatry
    #   (třeba @cached(expires=8000) ) a pak chci vrátit správně upravený
    #   ten cached objekt
    if isinstance(expires, FunctionType):
        # jedna se o prime volani jako funkce a ne jako trida
        return RedCached()(expires)
    # endif
    return RedCached(
        expires, conditionFunction, blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for, key=key, object_attr=attr,
        marshaller=marshaller
    )
# enddef cursor


def red_cached_json2(
    expires=3600, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache"
):
    return red_cached(
        expires=expires,
        conditionFunction=conditionFunction,
        blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for,
        key=key,
        attr=attr,
        marshaller=json2
    )
# enddef


def red_cached_rpc_raw_response(
    expires=3600, conditionFunction=None, blocking_call_for=None,
    waiting_call_for=None, key=None, attr="cache"
):
    return red_cached(
        expires=expires,
        conditionFunction=conditionFunction,
        blocking_call_for=blocking_call_for,
        waiting_call_for=waiting_call_for,
        key=key,
        attr=attr,
        marshaller=MarshallerJsonRpcRaw
    )
# enddef

# eof
