from .cached import Cached
from types import FunctionType

from logging import getLogger
log = getLogger("jsonrpc.decorators.uwsgi_cached")
try:
    import uwsgi
except ImportError:
    log.warning("Unable to use UWSGI CACHE")
    uwsgi = None
# endtry


class UwsgiCached(Cached):

    def __init__(self, **kwargs):
        cache_name = kwargs.pop("cache", None)
        kwargs["object_attr"] = None
        super(UwsgiCached, self).__init__(**kwargs)
        del(self.object_attr)
        if cache_name:
            self.cache_extra_args = (cache_name,)
        else:
            self.cache_extra_args = ()
    # enddef __init__

    def _set(self, cacheObject, key, val, time):
        time = time if time is not None else 3600
        try:
            return cacheObject.cache_update(
                key, val, time, *self.cache_extra_args
            )
        except BaseException:
            log.exception("Error writing to cache")
            raise

    def _delete(self, cacheObject, key):
        return cacheObject.cache_del(key, *self.cache_extra_args)

    def _get(self, cacheObject, key):
        v = cacheObject.cache_get(key, *self.cache_extra_args)
        if v is None:
            log.debug(
                "Not found %r in cache if its too often you have propably"
                "too small blocksize - consider rising it", key
            )
        return v

    def _get_cache_object(self, args):
        return uwsgi

# endclass Cached


def uwsgi_cached(*args, **kwargs):
    """
    dekorator pro cache
    """
    # pokud se jedna o volani bez parametru pomoci @cached tak je v expires
    #   ta funkce pokud ovšem ne tak se jedná o volání typu s paramatry
    #   (třeba @cached(expires=8000) ) a pak chci vrátit správně upravený
    #   ten cached objekt
    if args and isinstance(args[0], FunctionType):
        if len(args) == 1 and not kwargs:
            # jedna se o prime volani jako funkce a ne jako trida
            return UwsgiCached()(args[0])
        else:
            raise RuntimeError(
                "Initialization Error:args:%r-kwarsg:%r" % (args, kwargs)
            )
    # endif
    if len(args) == 1 and "expires" not in kwargs:
        kwargs["expires"] = args[0]
        args = ()
    if args:
        raise RuntimeError(
            "Initialization Error:args:%r-kwarsg:%r" % (args, kwargs)
        )

    return UwsgiCached(**kwargs)
# enddef cursor

# eof
