from checkerslib.custom_exceptions import BaseRPCException  # noqa [F401]
from checkerslib.custom_exceptions import RPCException
from checkerslib.custom_exceptions import WrongArgumentLength  # noqa [F401]
from checkerslib.custom_exceptions import MissingKey  # noqa [F401]
from checkerslib.custom_exceptions import CheckRegexValueFailed  # noqa [F401]


class Redirect(RPCException):
    status = 302
    http_status = 302
    status_message = "Redirect"

    def __init__(self, location):
        if isinstance(location, str):
            location = location.encode('utf-8')
        self.headers = {
            "Location": location,
            "Content-Type": "text/html"
        }
        self.content = """
        <html>
            <head><meta http-equiv="refresh" content="1;url=%s"></head>
            <body><a href="%s">Přesměrování</a></body>
        </html>"""
        self.content %= (location, location)


class MissingArgument(RPCException):
    status = 400
    http_status = 400
    status_message = "Missing argument %s: %r"

    def __init__(self, func_name, arg_name):
        self.status_message %= (func_name, arg_name)


class DontCallMeAgain(RPCException):
    status = 428
    http_status = 428
    status_message = "Please stop calling me like that. I will never be "\
        "capable of fulfill such request"


class Locked(RPCException):
    status = 423
    http_status = 423
    status_message = "The resource that is being acessed is locked"


class NotFound(RPCException):
    "Not found"
    status = 404
    http_status = 404
    status_message = "Not found"
    silent = True


class NoContent(RPCException):
    status = 204
    http_status = 204
    status_message = "No content"


class MethodNotFound(NotFound):
    def __init__(self, method_name):
        self.status_message = "RPC method %s not found" % method_name


class MethodHelpNotFound(NotFound):

    def __init__(self, method_name):
        self.status_message = "RPC method %s has no documentation"
        self.status_message %= method_name


class WrongArgument(RPCException):
    status = 400
    http_status = 400

    def __init__(self, arg_name, value_type, expected_type, history):
        self.status_message = (
            "Wrong value for argument '%s', expected '%s', received %r")
        self.status_message %= (arg_name, expected_type, value_type)
        if history:
            self.status_message += "in %s" % history


class UnknownArgument(RPCException):
    status = 400
    http_status = 400
    status_message = "Function %s received unknown parameter: %r, value: %r"

    def __init__(self, func_name, param, value):
        self.status_message %= (func_name, param, value)


class SessionRequired(RPCException):
    status = 403
    http_status = 403
    status_message = "Tato metoda vyzaduje session"


class PreconditionFailed(RPCException):
    status = 412
    http_status = 412
    status_message = "Precondition Failed"
