import os
import stat
from time import time
import datetime


def file_time(dir_path, oldest=True, mode="c"):
    if mode not in ("c", "a", "m"):
        raise RuntimeError("Last time mode can be only c/m/a")
    n = int(time())
    if oldest:
        t = 0
        fn = max
    else:
        t = n + 3600  # budoucnost
        fn = min
    for f in os.listdir(dir_path):
        f_path = os.path.join(dir_path, f)
        s = os.stat(f_path)
        if not stat.S_ISREG(s.st_mode):
            continue
        t = fn(t, n - getattr(s, "st_" + mode + "time"))
    # endfor
    return int(t) if t > 0 else 0
# enddef


def oldest_file_check(dir_path, mode="c"):
    t = file_time(dir_path, oldest=True, mode=mode)
    if t < 1:
        return {
            "status": 0,
            "status_message": "OK"
        }
    return {
        "status": t,
        "status_message": "Oldest file: %s" % (
            datetime.timedelta(seconds=t),
        )
    }


def newest_file_check(dir_path, mode="m"):
    t = file_time(dir_path, oldest=False, mode=mode)
    if t < 1:
        return {
            "status": 0,
            "status_message": "OK"
        }
    return {
        "status": t,
        "status_message": "Newest file: %s" % (
            datetime.timedelta(seconds=t),
        )
    }


if __name__ == '__main__':
    print(oldest_file_check("/tmp/"))
    print(oldest_file_check("/var/log/"))
    print(newest_file_check("/tmp/"))
    print(newest_file_check("/var/log"))
# endif

# eof
