import logging
import logging.handlers
import re
from multiprocessing import Lock
try:
    import uwsgi
except ImportError:
    uwsgi = None

whitespace_pattern = re.compile(r'([\s\+\-\(\)]+)')


def unify_msisdn(val, with_plus=False, as_number=False, only_cz=False):
    if isinstance(val, str):
        val = whitespace_pattern.sub("", val)  # pozor sub je cim-co
        val = val.lstrip("+")
    elif isinstance(val, int):
        val = str(val)
    else:
        raise ValueError("Not string or int")
    # endif

    if not val.isdigit():
        raise ValueError("Contains non digit chars")

    val = val.lstrip("0")
    vallen = len(val)

    if vallen == 9:
        val = "420" + val
    elif vallen >= 11 and vallen <= 13:
        pass
    else:
        raise ValueError("Wrong length %s" % (vallen, ))

    if only_cz and not val.startswith("420"):
        raise ValueError("Only czech numbers allowed")

    if with_plus:
        return "+" + val

    if as_number:
        return int(val)

    return val
# enddef


class LockingFileHandler(logging.handlers.WatchedFileHandler):

    def __init__(self, *args, **kwargs):
        logging.handlers.WatchedFileHandler.__init__(self, *args, **kwargs)
        self._lock = Lock()
    # enddef

    def emit(self, *args, **kwargs):
        self._lock.acquire()
        logging.handlers.WatchedFileHandler.emit(self, *args, **kwargs)
        self._lock.release()
    # enddef
# endclass


if uwsgi is not None:
    class UwsgiLockingFileHandler(logging.handlers.WatchedFileHandler):

        def __init__(self, *args, **kwargs):
            logging.handlers.WatchedFileHandler.__init__(self, *args, **kwargs)
        # enddef

        def emit(self, *args, **kwargs):
            if uwsgi.i_am_the_spooler():
                logging.handlers.WatchedFileHandler.emit(self, *args, **kwargs)
                return
            uwsgi.lock()
            try:
                logging.handlers.WatchedFileHandler.emit(self, *args, **kwargs)
            finally:
                uwsgi.unlock()
        # enddef
    # endclass
