#!/usr/bin/python
# -*- coding: utf-8 -*-
import traceback
from logging import getLogger
import types

wrap_log = getLogger("jsonrpc.wrappers")

cur_log = getLogger("jsonrpc.decorators.cursor")
cur_info_log = getLogger("jsonrpc.decorators.cursor_info")
cur_commmit_log = getLogger("jsonrpc.decorators.cursor.commit")


def make_response_obj(
    data, content_type=None, headers=None, http_code=200, data_type=None
):
    if isinstance(data, tuple(generatorTypes)):
        RpcRespObj = GeneratorRespObj
    else:
        class RpcRespObj(type(data)):
            content_type = None
            headers = None
            http_code = None
            data_type = None  # nebo "json"
        # endclass
    # endif
    wrap_log.debug(
        "Wrapping response object %s as %s", type(data), RpcRespObj
    )
    rpc_resp_obj = RpcRespObj(data)
    rpc_resp_obj.content_type = content_type
    rpc_resp_obj.headers = headers or []
    rpc_resp_obj.http_code = http_code
    rpc_resp_obj.data_type = data_type
    return rpc_resp_obj


class ResponseWrap(object):
    def __init__(self, sr_func):
        self.fnc = sr_func

    def __call__(self, sts, headers):
        self.fnc(sts, headers)
        self.fnc = self.empty

    def empty(self, sts, headers):
        pass


class CursorGeneratorWrapper(object):
    __data = None
    __iter = None
    __cursor = None
    __conn = None
    __reuse = None

    def __init__(self, cursor, generator, conn, reuse=False):
        wrap_log.debug("creating wrapper of %s", generator)
        self.__data = generator
        self.__iter = iter(generator)
        self.__cursor = cursor
        self.__conn = conn
        self.__reuse = reuse

    def __getattr__(self, attr):
        if attr in (
            "_CursorGeneratorWrapper__data", "_CursorGeneratorWrapper__cursor",
            "_CursorGeneratorWrapper__conn", "_CursorGeneratorWrapper__reuse",
            "_CursorGeneratorWrapper__iter",
        ):
            return object.__getattr__(self, attr)
        return getattr(self.__data, attr)

    def __setattr__(self, attr, val):
        if attr in (
            "_CursorGeneratorWrapper__data", "_CursorGeneratorWrapper__cursor",
            "_CursorGeneratorWrapper__conn", "_CursorGeneratorWrapper__reuse",
            "_CursorGeneratorWrapper__iter",
        ):
            return object.__setattr__(self, attr, val)
        return setattr(self.__data, attr, val)

    def __next__(self):
        try:
            return next(self.__iter)
        except StopIteration:
            if not self.__reuse:
                self.__cursor.close()
                cur_log.debug("COMMIT")
                cur_commmit_log.debug(
                    "Commit\n%s!", traceback.format_stack()
                )
                self.__conn.commit()

            raise StopIteration

    # py2 next vs py3 __next__
    # next = __next__

    def __iter__(self):
        return self


class GeneratorRespObj(object):
    content_type = None
    headers = None
    http_code = None
    data_type = None
    __data = None
    __iter = None

    def __init__(self, data):
        wrap_log.debug("creating wrapper of %s", data)
        self.__data = data
        self.__iter = iter(data)

    def __getattr__(self, attr):
        if attr in (
            "_GeneratorRespObj__data",
            "_GeneratorRespObj__iter",
            "content_type", "headers", "http_code", "data_type"
        ):
            return object.__getattr__(self, attr)
        return getattr(self.__data, attr)

    def __setattr__(self, attr, val):
        if attr in (
            "_GeneratorRespObj__data",
            "_GeneratorRespObj__iter",
            "content_type", "headers", "http_code", "data_type"
        ):
            return object.__setattr__(self, attr, val)
        return setattr(self.__data, attr, val)

    def __next__(self):
        return next(self.__iter)

    # py2 next vs py3 __next__
    # next = __next__

    def __iter__(self):
        return self

# endclass


generatorTypes = [
    types.GeneratorType,
    CursorGeneratorWrapper,
    GeneratorRespObj
]


# pokud chceme používat na venek i kurzory musíme s nimi pracovat jako
#   s generátory a ne všechny jsou => musíme si je virtuálně přidat
#   mezi generátory
def register_generator(g):
    if g not in generatorTypes:
        generatorTypes.append(g)
# eof
